@@ -1,14 +1,10 @@
 #!/bin/sh
-# $Id: testminissdpd.sh,v 1.6 2016/12/16 09:22:36 nanard Exp $
-# (c) 2016 Thomas Bernard
+# $Id: testminissdpd.sh,v 1.8 2017/04/21 11:57:59 nanard Exp $
+# (c) 2017 Thomas Bernard
 
 OS=`uname -s`
 IF=lo
-<<<<<<< testminissdpd.sh
-if [ "$OS" = "Darwin" ] || [ "$OS" = "OpenBSD" ] ; then
-=======
-if [ "$OS" = "Darwin" ] || [ "$OS" = "SunOS" ] ; then
->>>>>>> 1.6
+if [ "$OS" = "Darwin" ] || [ "$OS" = "OpenBSD" ] || [ "$OS" = "SunOS" ] || [ "$OS" = "FreeBSD" ] ; then
 	IF=lo0
 fi
 # if set, 1st argument is network interface
